# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

# PyQt
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
import matplotlib.animation as animation


class MatplotlibViewerCanvas(FigureCanvas):
    """
    Inherits from FigureCanvasQTAgg in order to integrate with PyQt.
    """

    def __init__(self, parent, signal_provider, period):

        # create a new figure
        self.fig = Figure(dpi=100)

        # call FigureCanvas constructor
        FigureCanvas.__init__(self, self.fig)

        # set the parent of this FigureCanvas
        self.setParent(parent)

        # set signal provider
        self.signal_provider = signal_provider

        # setup the plot and the animations
        self.index = 0
        # add plot to the figure
        self.axes = self.fig.add_subplot()
        # set axes labels
        self.axes.set_xlabel("time [s]")
        self.axes.set_ylabel("value")
        self.axes.grid(True)

        # start the vertical line animation
        (self.vertical_line,) = self.axes.plot([], [], "-", lw=1, c="k")

        self.periond_in_ms = int(period * 1000)

        self.vertical_line_anim = animation.FuncAnimation(
            self.fig,
            self.update_vertical_line,
            init_func=self.init_vertical_line,
            interval=self.periond_in_ms,
            blit=True,
        )

        # active paths
        self.active_paths = {}

        # add plot toolbar from matplotlib
        self.toolbar = NavigationToolbar(self, self)

    def quit_animation(self):
        # https://stackoverflow.com/questions/32280140/cannot-delete-matplotlib-animation-funcanimation-objects
        # this is to close the event associated to the animation

        # this is required with matplotlib 3.1.2 but not with 3.5.1.
        # However this code will run with both version of matplotlib
        if self.vertical_line_anim:
            self.vertical_line_anim._stop()

    def update_plots(self, paths, legends):

        for path, legend in zip(paths, legends):
            path_string = "/".join(path)
            legend_string = "/".join(legend[1:])

            if path_string not in self.active_paths.keys():

                data = self.signal_provider.data
                for key in path[:-1]:
                    data = data[key]
                try:
                    datapoints = data["data"][:, int(path[-1])]
                except IndexError:
                    # This happens in the case the variable is a scalar.
                    datapoints = data["data"][:]

                timestamps = data["timestamps"] - self.signal_provider.initial_time

                (self.active_paths[path_string],) = self.axes.plot(
                    timestamps, datapoints, label=legend_string
                )

        paths_to_be_canceled = []
        for active_path in self.active_paths.keys():

            path = active_path.split("/")

            if path not in paths:
                paths_to_be_canceled.append(active_path)

        for path in paths_to_be_canceled:
            self.active_paths[path].remove()
            self.active_paths.pop(path)

        self.axes.set_xlim(
            0, self.signal_provider.end_time - self.signal_provider.initial_time
        )

        # Since a new plot has been added/removed we delete the old animation and we create a new one
        # TODO: this part could be optimized
        self.vertical_line_anim._stop()
        self.axes.legend()
        self.vertical_line_anim = animation.FuncAnimation(
            self.fig,
            self.update_vertical_line,
            init_func=self.init_vertical_line,
            interval=self.periond_in_ms,
            blit=True,
        )

    def update_index(self, index):
        self.index = index

    def init_vertical_line(self):
        self.vertical_line.set_data([], [])
        return self.vertical_line, *(self.active_paths.values())

    def update_vertical_line(self, _):
        """
        Update the vertical line
        """
        current_time = self.signal_provider.current_time
        # Draw vertical line at current index

        self.vertical_line.set_data([current_time, current_time], self.axes.get_ylim())
        return self.vertical_line, *(self.active_paths.values())



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

from PyQt5.QtCore import QThread, QMutex, QMutexLocker

import icub_models

import numpy as np
import time

from robot_log_visualizer.robot_visualizer.meshcat_visualizer import MeshcatVisualizer

from robot_log_visualizer.utils.utils import PeriodicThreadState


class MeshcatProvider(QThread):
    def __init__(self, signal_provider, period):
        QThread.__init__(self)

        self._state = PeriodicThreadState.pause
        self.state_lock = QMutex()

        self._period = period
        self.meshcat_visualizer = MeshcatVisualizer()
        self._signal_provider = signal_provider

    @property
    def state(self):
        locker = QMutexLocker(self.state_lock)
        value = self._state
        return value

    @state.setter
    def state(self, new_state: PeriodicThreadState):
        locker = QMutexLocker(self.state_lock)
        self._state = new_state

    def load_model(self, considered_joints, model_name):
        if not model_name in icub_models.get_robot_names():
            model_name = "iCubGenova09"
        self.meshcat_visualizer.load_model_from_file(
            model_path=icub_models.get_model_file(model_name),
            considered_joints=considered_joints,
            model_name="robot",
        )

    def run(self):
        base_rotation = np.eye(3)
        base_position = np.array([0.0, 0.0, 0.0])

        while True:
            start = time.time()

            if self.state == PeriodicThreadState.running:

                # These are the robot measured joint positions in radians
                joints = self._signal_provider.data[self._signal_provider.root_name][
                    "joints_state"
                ]["positions"]["data"]

                self.meshcat_visualizer.set_multy_body_system_state(
                    base_position,
                    base_rotation,
                    joint_value=joints[self._signal_provider.index, :],
                    model_name="robot",
                )

            sleep_time = self._period - (time.time() - start)
            if sleep_time > 0:
                time.sleep(sleep_time)

            if self.state == PeriodicThreadState.closed:
                return



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

from PyQt5.QtCore import QUrl
from PyQt5.QtWidgets import QFrame
from PyQt5.QtMultimedia import QMediaContent, QMediaPlayer

from robot_log_visualizer.ui.autogenerated.video_tab import Ui_VideoTab
import os


class VideoItem(QFrame):
    def __init__(self, video_filename: str):
        super().__init__(None)
        self.ui = Ui_VideoTab()
        self.ui.setupUi(self)

        self.media_player = QMediaPlayer(None, QMediaPlayer.VideoSurface)
        self.media_player.setVideoOutput(self.ui.webcamView)

        self.media_loaded = False

        if os.path.isfile(video_filename):
            self.media_player.setMedia(
                QMediaContent(QUrl.fromLocalFile(video_filename))
            )
            self.media_loaded = True



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'robot_log_visualizer/ui/misc/visualizer.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1073, 908)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter_2 = QtWidgets.QSplitter(self.centralwidget)
        self.splitter_2.setOrientation(QtCore.Qt.Vertical)
        self.splitter_2.setChildrenCollapsible(False)
        self.splitter_2.setObjectName("splitter_2")
        self.dataVisualizationFrame = QtWidgets.QFrame(self.splitter_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(2)
        sizePolicy.setHeightForWidth(self.dataVisualizationFrame.sizePolicy().hasHeightForWidth())
        self.dataVisualizationFrame.setSizePolicy(sizePolicy)
        self.dataVisualizationFrame.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.dataVisualizationFrame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.dataVisualizationFrame.setObjectName("dataVisualizationFrame")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.dataVisualizationFrame)
        self.gridLayout_3.setContentsMargins(-1, 9, -1, -1)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.pauseButton = QtWidgets.QPushButton(self.dataVisualizationFrame)
        self.pauseButton.setEnabled(False)
        self.pauseButton.setMaximumSize(QtCore.QSize(40, 16777215))
        self.pauseButton.setText("")
        icon = QtGui.QIcon.fromTheme("media-playback-pause")
        self.pauseButton.setIcon(icon)
        self.pauseButton.setCheckable(False)
        self.pauseButton.setDefault(False)
        self.pauseButton.setFlat(False)
        self.pauseButton.setObjectName("pauseButton")
        self.gridLayout_3.addWidget(self.pauseButton, 1, 4, 1, 1)
        self.timeSlider = QtWidgets.QSlider(self.dataVisualizationFrame)
        self.timeSlider.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeSlider.sizePolicy().hasHeightForWidth())
        self.timeSlider.setSizePolicy(sizePolicy)
        self.timeSlider.setPageStep(1)
        self.timeSlider.setTracking(True)
        self.timeSlider.setOrientation(QtCore.Qt.Horizontal)
        self.timeSlider.setObjectName("timeSlider")
        self.gridLayout_3.addWidget(self.timeSlider, 1, 1, 1, 2)
        self.startButton = QtWidgets.QPushButton(self.dataVisualizationFrame)
        self.startButton.setEnabled(False)
        self.startButton.setMaximumSize(QtCore.QSize(40, 16777215))
        self.startButton.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.startButton.setText("")
        icon = QtGui.QIcon.fromTheme("media-playback-start")
        self.startButton.setIcon(icon)
        self.startButton.setObjectName("startButton")
        self.gridLayout_3.addWidget(self.startButton, 1, 3, 1, 1)
        self.timeLabel = QtWidgets.QLabel(self.dataVisualizationFrame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.timeLabel.sizePolicy().hasHeightForWidth())
        self.timeLabel.setSizePolicy(sizePolicy)
        self.timeLabel.setMinimumSize(QtCore.QSize(60, 0))
        self.timeLabel.setObjectName("timeLabel")
        self.gridLayout_3.addWidget(self.timeLabel, 1, 0, 1, 1)
        self.splitter = QtWidgets.QSplitter(self.dataVisualizationFrame)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setChildrenCollapsible(True)
        self.splitter.setObjectName("splitter")
        self.variableTreeWidget = QtWidgets.QTreeWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.variableTreeWidget.sizePolicy().hasHeightForWidth())
        self.variableTreeWidget.setSizePolicy(sizePolicy)
        self.variableTreeWidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.variableTreeWidget.setSizeIncrement(QtCore.QSize(0, 0))
        self.variableTreeWidget.setBaseSize(QtCore.QSize(0, 0))
        self.variableTreeWidget.setFrameShape(QtWidgets.QFrame.Box)
        self.variableTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.variableTreeWidget.setAnimated(True)
        self.variableTreeWidget.setObjectName("variableTreeWidget")
        self.variableTreeWidget.header().setVisible(False)
        self.variableTreeWidget.header().setDefaultSectionSize(100)
        self.tabPlotWidget = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabPlotWidget.sizePolicy().hasHeightForWidth())
        self.tabPlotWidget.setSizePolicy(sizePolicy)
        self.tabPlotWidget.setTabsClosable(True)
        self.tabPlotWidget.setMovable(False)
        self.tabPlotWidget.setTabBarAutoHide(False)
        self.tabPlotWidget.setObjectName("tabPlotWidget")
        self.meshcatAndVideoTab = QtWidgets.QTabWidget(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.meshcatAndVideoTab.sizePolicy().hasHeightForWidth())
        self.meshcatAndVideoTab.setSizePolicy(sizePolicy)
        self.meshcatAndVideoTab.setTabPosition(QtWidgets.QTabWidget.East)
        self.meshcatAndVideoTab.setDocumentMode(False)
        self.meshcatAndVideoTab.setObjectName("meshcatAndVideoTab")
        self.meshcatTab = QtWidgets.QWidget()
        self.meshcatTab.setObjectName("meshcatTab")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.meshcatTab)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.meshcatView = QtWebEngineWidgets.QWebEngineView(self.meshcatTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.meshcatView.sizePolicy().hasHeightForWidth())
        self.meshcatView.setSizePolicy(sizePolicy)
        self.meshcatView.setObjectName("meshcatView")
        self.horizontalLayout.addWidget(self.meshcatView)
        icon = QtGui.QIcon.fromTheme("input-gaming")
        self.meshcatAndVideoTab.addTab(self.meshcatTab, icon, "")
        self.gridLayout_3.addWidget(self.splitter, 0, 0, 1, 5)
        self.tabWidget = QtWidgets.QTabWidget(self.splitter_2)
        self.tabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.tabWidget.setAutoFillBackground(False)
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabWidget.setUsesScrollButtons(False)
        self.tabWidget.setDocumentMode(False)
        self.tabWidget.setTabsClosable(False)
        self.tabWidget.setMovable(True)
        self.tabWidget.setObjectName("tabWidget")
        self.yarpTextLogWidget = QtWidgets.QWidget()
        self.yarpTextLogWidget.setObjectName("yarpTextLogWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.yarpTextLogWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.splitter_3 = QtWidgets.QSplitter(self.yarpTextLogWidget)
        self.splitter_3.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_3.setObjectName("splitter_3")
        self.yarpTextLogTreeWidget = QtWidgets.QTreeWidget(self.splitter_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.yarpTextLogTreeWidget.sizePolicy().hasHeightForWidth())
        self.yarpTextLogTreeWidget.setSizePolicy(sizePolicy)
        self.yarpTextLogTreeWidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.yarpTextLogTreeWidget.setSizeIncrement(QtCore.QSize(0, 0))
        self.yarpTextLogTreeWidget.setBaseSize(QtCore.QSize(0, 0))
        self.yarpTextLogTreeWidget.setFrameShape(QtWidgets.QFrame.Box)
        self.yarpTextLogTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.yarpTextLogTreeWidget.setAnimated(True)
        self.yarpTextLogTreeWidget.setObjectName("yarpTextLogTreeWidget")
        self.yarpTextLogTreeWidget.header().setVisible(False)
        self.yarpTextLogTreeWidget.header().setDefaultSectionSize(100)
        self.yarpTextLogTableWidget = QtWidgets.QTableWidget(self.splitter_3)
        self.yarpTextLogTableWidget.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.yarpTextLogTableWidget.sizePolicy().hasHeightForWidth())
        self.yarpTextLogTableWidget.setSizePolicy(sizePolicy)
        self.yarpTextLogTableWidget.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.yarpTextLogTableWidget.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustIgnored)
        self.yarpTextLogTableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.yarpTextLogTableWidget.setSelectionMode(QtWidgets.QAbstractItemView.NoSelection)
        self.yarpTextLogTableWidget.setHorizontalScrollMode(QtWidgets.QAbstractItemView.ScrollPerPixel)
        self.yarpTextLogTableWidget.setShowGrid(False)
        self.yarpTextLogTableWidget.setGridStyle(QtCore.Qt.NoPen)
        self.yarpTextLogTableWidget.setObjectName("yarpTextLogTableWidget")
        self.yarpTextLogTableWidget.setColumnCount(0)
        self.yarpTextLogTableWidget.setRowCount(0)
        self.yarpTextLogTableWidget.horizontalHeader().setVisible(False)
        self.yarpTextLogTableWidget.verticalHeader().setVisible(False)
        self.verticalLayout_3.addWidget(self.splitter_3)
        icon = QtGui.QIcon.fromTheme("zoom-in")
        self.tabWidget.addTab(self.yarpTextLogWidget, icon, "")
        self.pythonWidget = QtWidgets.QWidget()
        self.pythonWidget.setAutoFillBackground(False)
        self.pythonWidget.setObjectName("pythonWidget")
        self.pythonWidgetLayout = QtWidgets.QVBoxLayout(self.pythonWidget)
        self.pythonWidgetLayout.setObjectName("pythonWidgetLayout")
        icon = QtGui.QIcon.fromTheme("terminal")
        self.tabWidget.addTab(self.pythonWidget, icon, "")
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab_5)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.logScrollArea = QtWidgets.QScrollArea(self.tab_5)
        self.logScrollArea.setMinimumSize(QtCore.QSize(0, 120))
        self.logScrollArea.setAutoFillBackground(False)
        self.logScrollArea.setStyleSheet("background: white")
        self.logScrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.logScrollArea.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.logScrollArea.setWidgetResizable(True)
        self.logScrollArea.setObjectName("logScrollArea")
        self.logScrollAreaWidgetContents = QtWidgets.QWidget()
        self.logScrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 1004, 190))
        self.logScrollAreaWidgetContents.setObjectName("logScrollAreaWidgetContents")
        self.gridLayout = QtWidgets.QGridLayout(self.logScrollAreaWidgetContents)
        self.gridLayout.setObjectName("gridLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 2, 0, 1, 1)
        self.logLabel = QtWidgets.QLabel(self.logScrollAreaWidgetContents)
        self.logLabel.setCursor(QtGui.QCursor(QtCore.Qt.ArrowCursor))
        self.logLabel.setText("")
        self.logLabel.setObjectName("logLabel")
        self.gridLayout.addWidget(self.logLabel, 1, 0, 1, 1)
        self.logScrollArea.setWidget(self.logScrollAreaWidgetContents)
        self.verticalLayout_2.addWidget(self.logScrollArea)
        icon = QtGui.QIcon.fromTheme("document")
        self.tabWidget.addTab(self.tab_5, icon, "")
        self.verticalLayout.addWidget(self.splitter_2)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1073, 22))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.actionQuit = QtWidgets.QAction(MainWindow)
        icon = QtGui.QIcon.fromTheme("exit")
        self.actionQuit.setIcon(icon)
        self.actionQuit.setObjectName("actionQuit")
        self.actionOpen = QtWidgets.QAction(MainWindow)
        icon = QtGui.QIcon.fromTheme("document-open")
        self.actionOpen.setIcon(icon)
        self.actionOpen.setObjectName("actionOpen")
        self.actionAbout = QtWidgets.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabPlotWidget.setCurrentIndex(-1)
        self.meshcatAndVideoTab.setCurrentIndex(0)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "Robot Log Visualizer"))
        self.timeLabel.setText(_translate("MainWindow", "0.0"))
        self.variableTreeWidget.headerItem().setText(0, _translate("MainWindow", "Variables"))
        self.yarpTextLogTreeWidget.headerItem().setText(0, _translate("MainWindow", "Variables"))
        self.yarpTextLogTableWidget.setSortingEnabled(False)
        self.menuFile.setTitle(_translate("MainWindow", "&File"))
        self.menuHelp.setTitle(_translate("MainWindow", "Help"))
        self.actionQuit.setText(_translate("MainWindow", "&Quit"))
        self.actionQuit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionOpen.setText(_translate("MainWindow", "&Open"))
        self.actionOpen.setShortcut(_translate("MainWindow", "Ctrl+O"))
        self.actionAbout.setText(_translate("MainWindow", "About"))
from PyQt5 import QtWebEngineWidgets



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'robot_log_visualizer/ui/misc/about.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_aboutWindow(object):
    def setupUi(self, aboutWindow):
        aboutWindow.setObjectName("aboutWindow")
        aboutWindow.resize(541, 102)
        aboutWindow.setMaximumSize(QtCore.QSize(541, 102))
        self.centralwidget = QtWidgets.QWidget(aboutWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label_2 = QtWidgets.QLabel(self.centralwidget)
        self.label_2.setEnabled(True)
        self.label_2.setOpenExternalLinks(True)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout.addWidget(self.label_2)
        aboutWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(aboutWindow)
        QtCore.QMetaObject.connectSlotsByName(aboutWindow)

    def retranslateUi(self, aboutWindow):
        _translate = QtCore.QCoreApplication.translate
        aboutWindow.setWindowTitle(_translate("aboutWindow", "Robot Log Visualizer - About"))
        self.label_2.setText(_translate("aboutWindow", "<html><head/><body><p><span style=\" font-size:14pt; font-weight:600;\">About Robot Log Visualizer</span></p><p><span style=\" font-size:12pt;\">This program is Licensed under the </span><a href=\"https://github.com/ami-iit/robot-log-visualizer/blob/main/LICENSE\"><span style=\" font-size:12pt; text-decoration: underline; color:#0000ff;\">3-Clause BSD License</span></a></p><p><span style=\" font-size:12pt;\">The project is mantained by the </span><a href=\"https://ami.iit.it/\"><span style=\" font-size:12pt; text-decoration: underline; color:#0000ff;\">Artificial and Mechanical Intelligence lab</span></a></p></body></html>"))



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

from enum import Enum


class PeriodicThreadState(Enum):
    running = (0,)
    pause = (1,)
    closed = 2



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

import time
import math
import h5py
import numpy as np
from PyQt5.QtCore import pyqtSignal, QThread, QMutex, QMutexLocker
from robot_log_visualizer.utils.utils import PeriodicThreadState


class TextLoggingMsg:
    def __init__(self, level, text):
        self.level = level
        self.text = text

    def color(self):
        if self.level == "ERROR":
            return "#d62728"
        elif self.level == "WARNING":
            return "#ff7f0e"
        elif self.level == "DEBUG":
            return "#1f77b4"
        elif self.level == "INFO":
            return "#2ca02c"
        else:
            return "black"


class SignalProvider(QThread):
    update_index_signal = pyqtSignal()

    def __init__(self, period: float):
        QThread.__init__(self)

        # set device state
        self._state = PeriodicThreadState.pause
        self.state_lock = QMutex()

        self._index = 0
        self.index_lock = QMutex()

        self.period = period

        self.data = {}
        self.timestamps = np.array([])
        self.text_logging_data = {}

        self.initial_time = math.inf
        self.end_time = -math.inf

        self.joints_name = []
        self.robot_name = ""

        self.root_name = "robot_logger_device"

        self._current_time = 0

    def __populate_text_logging_data(self, file_object):
        data = {}
        for key, value in file_object.items():
            if not isinstance(value, h5py._hl.group.Group):
                continue
            if key == "#refs#":
                continue
            if "data" in value.keys():
                data[key] = {}
                level_ref = value["data"]["level"]
                text_ref = value["data"]["text"]

                data[key]["data"] = [
                    TextLoggingMsg(
                        text="".join(chr(c[0]) for c in value[text]),
                        level="".join(chr(c[0]) for c in value[level]),
                    )
                    for text, level in zip(text_ref[0], level_ref[0])
                ]

                data[key]["timestamps"] = np.squeeze(np.array(value["timestamps"]))

            else:
                data[key] = self.__populate_text_logging_data(file_object=value)

        return data

    def __populate_numerical_data(self, file_object):
        data = {}
        for key, value in file_object.items():
            if not isinstance(value, h5py._hl.group.Group):
                continue
            if key == "#refs#":
                continue
            if key == "log":
                continue
            if "data" in value.keys():
                data[key] = {}
                data[key]["data"] = np.squeeze(np.array(value["data"]))
                data[key]["timestamps"] = np.squeeze(np.array(value["timestamps"]))

                # if the initial or end time has been updated we can also update the entire timestamps dataset
                if data[key]["timestamps"][0] < self.initial_time:
                    self.timestamps = data[key]["timestamps"]
                    self.initial_time = self.timestamps[0]

                if data[key]["timestamps"][-1] > self.end_time:
                    self.timestamps = data[key]["timestamps"]
                    self.end_time = self.timestamps[-1]

                # In yarp telemetry v0.4.0 the elements_names was saved.
                if "elements_names" in value.keys():
                    elements_names_ref = value["elements_names"]
                    data[key]["elements_names"] = [
                        "".join(chr(c[0]) for c in value[ref])
                        for ref in elements_names_ref[0]
                    ]
            else:
                data[key] = self.__populate_numerical_data(file_object=value)

        return data

    def open_mat_file(self, file_name: str):
        with h5py.File(file_name, "r") as file:
            root_variable = file.get(self.root_name)
            self.data = self.__populate_numerical_data(file)

            if "log" in root_variable.keys():
                self.text_logging_data["log"] = self.__populate_text_logging_data(
                    root_variable["log"]
                )

            for name in file.keys():
                if "description_list" in file[name].keys():
                    self.root_name = name
                    break

            joint_ref = root_variable["description_list"]
            self.joints_name = [
                "".join(chr(c[0]) for c in file[ref]) for ref in joint_ref[0]
            ]
            if "yarp_robot_name" in root_variable.keys():
                robot_name_ref = root_variable["yarp_robot_name"]
                try:
                    self.robot_name = "".join(chr(c[0]) for c in robot_name_ref)
                except:
                    pass
            self.index = 0

    def __len__(self):
        return self.timestamps.shape[0]

    @property
    def state(self):
        locker = QMutexLocker(self.state_lock)
        value = self._state
        return value

    @state.setter
    def state(self, new_state: PeriodicThreadState):
        locker = QMutexLocker(self.state_lock)
        self._state = new_state

    @property
    def index(self):
        locker = QMutexLocker(self.index_lock)
        value = self._index
        return value

    @index.setter
    def index(self, index):
        locker = QMutexLocker(self.index_lock)
        self._index = index

    def register_update_index(self, slot):
        self.update_index_signal.connect(slot)

    def update_index(self, index):
        locker = QMutexLocker(self.index_lock)
        self._index = max(min(index, len(self.timestamps) - 1), 0)
        self._current_time = self.timestamps[self._index] - self.initial_time

    @property
    def current_time(self):
        locker = QMutexLocker(self.index_lock)
        value = self._current_time
        return value

    def run(self):

        while True:
            start = time.time()
            if self.state == PeriodicThreadState.running:
                self.index_lock.lock()
                tmp_index = self._index
                # check if index must be increased
                if self._current_time > self.timestamps[tmp_index] - self.initial_time:
                    tmp_index += 1
                    tmp_index = min(tmp_index, self.timestamps.shape[0] - 1)

                self._index = tmp_index
                self._current_time += self.period
                self.index_lock.unlock()

                self.update_index_signal.emit()

            sleep_time = self.period - (time.time() - start)
            if sleep_time > 0:
                time.sleep(sleep_time)

            if self.state == PeriodicThreadState.closed:
                return



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'robot_log_visualizer/ui/misc/video_tab.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VideoTab(object):
    def setupUi(self, VideoTab):
        VideoTab.setObjectName("VideoTab")
        VideoTab.resize(400, 300)
        self.horizontalLayout = QtWidgets.QHBoxLayout(VideoTab)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.webcamView = QVideoWidget(VideoTab)
        self.webcamView.setObjectName("webcamView")
        self.horizontalLayout.addWidget(self.webcamView)

        self.retranslateUi(VideoTab)
        QtCore.QMetaObject.connectSlotsByName(VideoTab)

    def retranslateUi(self, VideoTab):
        _translate = QtCore.QCoreApplication.translate
        VideoTab.setWindowTitle(_translate("VideoTab", "Form"))
from PyQt5.QtMultimediaWidgets import QVideoWidget



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

from PyQt5.QtWidgets import QFrame

from robot_log_visualizer.ui.autogenerated.plot_tab import Ui_PlotTab
from robot_log_visualizer.plotter.matplotlib_viewer_canvas import MatplotlibViewerCanvas


class PlotItem(QFrame):
    def __init__(self, signal_provider, period):
        super().__init__(None)
        self.ui = Ui_PlotTab()
        self.ui.setupUi(self)

        self.canvas = MatplotlibViewerCanvas(
            parent=self, period=period, signal_provider=signal_provider
        )
        self.ui.plotLayout.addWidget(self.canvas)



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

# PyQt5
from PyQt5 import QtWidgets, QtGui
from PyQt5.QtCore import QUrl
from PyQt5.QtCore import pyqtSlot, Qt, QMutex, QMutexLocker
from PyQt5.QtWidgets import (
    QFileDialog,
    QTreeWidgetItem,
    QToolButton,
    QDialog,
    QVBoxLayout,
    QLineEdit,
    QDialogButtonBox,
    QTableWidgetItem,
)
from PyQt5.QtMultimedia import QMediaContent, QMediaPlayer

from robot_log_visualizer.ui.plot_item import PlotItem
from robot_log_visualizer.ui.video_item import VideoItem
from robot_log_visualizer.ui.text_logging import TextLoggingItem

from robot_log_visualizer.utils.utils import PeriodicThreadState

import sys
import os
import pathlib
import re

import numpy as np

# QtDesigner generated classes
from robot_log_visualizer.ui.autogenerated.visualizer import Ui_MainWindow
from robot_log_visualizer.ui.autogenerated.about import Ui_aboutWindow

# for logging
from time import localtime, strftime

# Matplotlib class
from pyqtconsole.console import PythonConsole
import pyqtconsole.highlighter as hl


class About(QtWidgets.QMainWindow):
    def __init__(self):
        # call QMainWindow constructor
        super().__init__()
        self.ui = Ui_aboutWindow()
        self.ui.setupUi(self)


def build_plot_title_box_dialog():
    dlg = QDialog()
    dlg.setWindowTitle("Plot title")
    la = QVBoxLayout(dlg)
    line_edit = QLineEdit()
    la.addWidget(line_edit)
    bb = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
    bb.clicked.connect(dlg.accept)
    bb.rejected.connect(dlg.reject)
    la.addWidget(bb)
    dlg.setLayout(la)
    return dlg, line_edit


def get_icon(icon_name):

    icon = QtGui.QIcon()
    icon.addPixmap(
        QtGui.QPixmap(str(pathlib.Path(__file__).parent / "misc" / icon_name)),
        QtGui.QIcon.Normal,
        QtGui.QIcon.Off,
    )
    return icon


class RobotViewerMainWindow(QtWidgets.QMainWindow):
    def __init__(self, signal_provider, meshcat_provider, animation_period):
        # call QMainWindow constructor
        super().__init__()

        self.animation_period = animation_period

        # set up the user interface
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)

        # Set all the icons
        self.ui.startButton.setIcon(get_icon("play-outline.svg"))
        self.ui.pauseButton.setIcon(get_icon("pause-outline.svg"))
        self.ui.pauseButton.setIcon(get_icon("pause-outline.svg"))
        self.ui.meshcatAndVideoTab.setTabIcon(
            0, get_icon("game-controller-outline.svg")
        )
        self.ui.tabWidget.setTabIcon(0, get_icon("calendar-outline.svg"))
        self.ui.tabWidget.setTabIcon(1, get_icon("terminal-outline.svg"))
        self.ui.tabWidget.setTabIcon(2, get_icon("document-text-outline.svg"))

        self.ui.actionQuit.setIcon(get_icon("close-circle-outline.svg"))
        self.ui.actionQuit.setIcon(get_icon("close-circle-outline.svg"))
        self.ui.actionOpen.setIcon(get_icon("folder-open-outline.svg"))
        self.setWindowIcon(get_icon("icon.png"))

        self.about = About()

        self.signal_provider = signal_provider
        self.signal_size = len(self.signal_provider)
        self.signal_provider.register_update_index(self.update_slider)

        self.meshcat_provider = meshcat_provider

        self.tool_button = QToolButton()
        self.tool_button.setText("+")
        self.ui.tabPlotWidget.setCornerWidget(self.tool_button)
        self.tool_button.clicked.connect(self.toolButton_on_click)

        self.plot_items = []
        self.video_items = []

        self.toolButton_on_click()

        # instantiate the Logger
        self.logger = Logger(self.ui.logLabel, self.ui.logScrollArea)
        # print welcome message
        self.logger.write_to_log("Robot Viewer started.")

        self.text_logger = TextLoggingItem(self.ui.yarpTextLogTableWidget)

        self._slider_pressed_mutex = QMutex()
        self._slider_pressed = False

        self.dataset_loaded = False

        # connect action
        self.ui.actionQuit.triggered.connect(self.close)
        self.ui.actionOpen.triggered.connect(self.open_mat_file)
        self.ui.actionAbout.triggered.connect(self.open_about)

        self.ui.meshcatView.setUrl(
            QUrl(meshcat_provider.meshcat_visualizer.viewer.url())
        )

        self.ui.pauseButton.clicked.connect(self.pauseButton_on_click)
        self.ui.startButton.clicked.connect(self.startButton_on_click)
        self.ui.timeSlider.sliderReleased.connect(self.timeSlider_on_release)
        self.ui.timeSlider.sliderPressed.connect(self.timeSlider_on_pressed)
        self.ui.timeSlider.sliderMoved.connect(self.timeSlider_on_sliderMoved)

        self.ui.variableTreeWidget.itemClicked.connect(self.variableTreeWidget_on_click)
        self.ui.yarpTextLogTreeWidget.itemClicked.connect(
            self.textLogTreeWidget_on_click
        )

        self.ui.tabPlotWidget.tabCloseRequested.connect(
            self.plotTabCloseButton_on_click
        )
        self.ui.tabPlotWidget.tabBarDoubleClicked.connect(
            self.plotTabBar_on_doubleClick
        )
        self.ui.tabPlotWidget.currentChanged.connect(self.plotTabBar_currentChanged)

        self.pyconsole = PythonConsole(
            parent=self.ui.pythonWidget,
            formats={
                "keyword": hl.format("#204a87", "bold"),
                "operator": hl.format("#ce5c00"),
                # 'brace':      hl.format('#eeeeec'),
                "defclass": hl.format("#000000", "bold"),
                "string": hl.format("#8f5902"),
                "string2": hl.format("#8f5902"),
                "comment": hl.format("#8f5902", "italic"),
                "self": hl.format("#000000", "italic"),
                "numbers": hl.format("#0000cf"),
                "inprompt": hl.format("#8f5902", "bold"),
                "outprompt": hl.format("#8f5902", "bold"),
            },
        )
        self.pyconsole.edit.setStyleSheet("font-size: 12px;")
        self.ui.pythonWidgetLayout.addWidget(self.pyconsole)
        self.pyconsole.eval_in_thread()

        # self.media_player = QMediaPlayer(None, QMediaPlayer.VideoSurface)
        # self.media_player.setVideoOutput(self.ui.webcamView)
        # self.media_loaded = False

    @property
    def slider_pressed(self):
        locker = QMutexLocker(self._slider_pressed_mutex)
        value = self._slider_pressed
        return value

    @slider_pressed.setter
    def slider_pressed(self, slider_pressed):
        locker = QMutexLocker(self._slider_pressed_mutex)
        self._slider_pressed = slider_pressed

    def keyPressEvent(self, event):
        if not self.dataset_loaded:
            return

        if event.modifiers() & Qt.ControlModifier:
            if event.key() == Qt.Key_B:
                self.slider_pressed = True
                new_index = int(self.ui.timeSlider.value()) - 1
                self.signal_provider.update_index(new_index)
                self.ui.timeLabel.setText(f"{self.signal_provider.current_time:.2f}")
                self.text_logger.highlight_cell(
                    self.find_text_log_index(self.get_text_log_item_path())
                )

                # for every video item we set the instant
                for video_item in self.video_items:
                    if video_item.media_loaded:
                        video_item.media_player.setPosition(
                            new_index
                            / self.ui.timeSlider.maximum()
                            * video_item.media_player.duration()
                        )

                # update the time slider
                self.ui.timeSlider.setValue(new_index)
                self.slider_pressed = False
            elif event.key() == Qt.Key_F:
                self.slider_pressed = True
                new_index = int(self.ui.timeSlider.value()) + 1
                self.signal_provider.update_index(new_index)
                self.ui.timeLabel.setText(f"{self.signal_provider.current_time:.2f}")
                self.text_logger.highlight_cell(
                    self.find_text_log_index(self.get_text_log_item_path())
                )

                # for every video item we set the instant
                for video_item in self.video_items:
                    if video_item.media_loaded:
                        video_item.media_player.setPosition(
                            new_index
                            / self.ui.timeSlider.maximum()
                            * video_item.media_player.duration()
                        )

                self.ui.timeSlider.setValue(new_index)
                self.slider_pressed = False

    def toolButton_on_click(self):
        self.plot_items.append(
            PlotItem(signal_provider=self.signal_provider, period=self.animation_period)
        )
        self.ui.tabPlotWidget.addTab(self.plot_items[-1], "Plot")

        if self.ui.tabPlotWidget.count() == 1:
            self.ui.tabPlotWidget.setTabsClosable(False)
        else:
            self.ui.tabPlotWidget.setTabsClosable(True)

    def timeSlider_on_pressed(self):
        self.slider_pressed = True

    def timeSlider_on_sliderMoved(self):
        index = int(self.ui.timeSlider.value())

        for video_item in self.video_items:
            if video_item.media_loaded:
                video_item.media_player.setPosition(
                    index
                    / self.ui.timeSlider.maximum()
                    * video_item.media_player.duration()
                )

        self.signal_provider.update_index(index)
        self.ui.timeLabel.setText(f"{self.signal_provider.current_time:.2f}")
        self.text_logger.highlight_cell(
            self.find_text_log_index(self.get_text_log_item_path())
        )

    def timeSlider_on_release(self):
        index = int(self.ui.timeSlider.value())

        for video_item in self.video_items:
            if video_item.media_loaded:
                video_item.media_player.setPosition(
                    index
                    / self.ui.timeSlider.maximum()
                    * video_item.media_player.duration()
                )

        self.signal_provider.update_index(index)
        self.ui.timeLabel.setText(f"{self.signal_provider.current_time:.2f}")
        self.text_logger.highlight_cell(
            self.find_text_log_index(self.get_text_log_item_path())
        )
        self.slider_pressed = False

    def startButton_on_click(self):
        self.ui.startButton.setEnabled(False)
        self.ui.pauseButton.setEnabled(True)

        ## TODO its better to start the video only if the tab is active. This will improve the peroformances
        for video_item in self.video_items:
            if video_item.media_loaded:
                video_item.media_player.play()

        self.signal_provider.state = PeriodicThreadState.running
        # self.meshcat_provider.state = PeriodicThreadState.running

        self.logger.write_to_log("Dataset started.")

    def pauseButton_on_click(self):
        self.ui.pauseButton.setEnabled(False)
        self.ui.startButton.setEnabled(True)

        for video_item in self.video_items:
            if video_item.media_loaded:
                video_item.media_player.pause()

        self.signal_provider.state = PeriodicThreadState.pause
        # self.meshcat_provider.state = PeriodicThreadState.pause

        self.logger.write_to_log("Dataset paused.")

    def plotTabCloseButton_on_click(self, index):

        self.ui.tabPlotWidget.removeTab(index)
        self.plot_items[index].canvas.quit_animation()
        del self.plot_items[index]

        if self.ui.tabPlotWidget.count() == 1:
            self.ui.tabPlotWidget.setTabsClosable(False)

    def plotTabBar_on_doubleClick(self, index):

        dlg, plot_title = build_plot_title_box_dialog()
        if dlg.exec() == QDialog.Accepted:
            self.ui.tabPlotWidget.setTabText(index, plot_title.text())

    def variableTreeWidget_on_click(self):
        paths = []
        legends = []
        for index in self.ui.variableTreeWidget.selectedIndexes():
            path = []
            legend = []
            is_leaf = True
            while index.data() is not None:
                legend.append(index.data())
                if not is_leaf:
                    path.append(index.data())
                else:
                    path.append(str(index.row()))
                    is_leaf = False

                index = index.parent()
            path.reverse()
            legend.reverse()

            paths.append(path)
            legends.append(legend)
        self.plot_items[self.ui.tabPlotWidget.currentIndex()].canvas.update_plots(
            paths, legends
        )

    def find_text_log_index(self, path):
        current_time = self.signal_provider.current_time
        initial_time = self.signal_provider.initial_time

        path = self.get_text_log_item_path()
        if path:
            ref = self.signal_provider.text_logging_data
            for key in path:
                ref = ref[key]

            s = np.flatnonzero(ref["timestamps"] <= current_time + initial_time)
            if len(s) == 1:
                return s[0]
            elif not s.any():
                return None
            else:
                return s[-1]

        return None

    def show_text_log(self, path):
        initial_time = self.signal_provider.initial_time
        ref = self.signal_provider.text_logging_data
        for key in path:
            ref = ref[key]

        self.text_logger.clean()
        logs = ref["data"]
        timestamps = ref["timestamps"]
        for i in range(len(ref["data"])):
            self.text_logger.add_entry(
                logs[i].text, timestamps[i] - initial_time, font_color=logs[i].color()
            )

    def get_text_log_item_path(self):
        paths = []
        for index in self.ui.yarpTextLogTreeWidget.selectedIndexes():
            path = []
            is_leaf = True
            while index.data() is not None:
                if not is_leaf:
                    path.append(index.data())
                else:
                    path.append(index.data())
                    is_leaf = False

                index = index.parent()

            path.reverse()
            paths.append(path)
        if paths:
            return paths[0]
        else:
            return None

    def textLogTreeWidget_on_click(self):
        path = self.get_text_log_item_path()
        if path:
            self.show_text_log(path)
            self.text_logger.highlight_cell(self.find_text_log_index(path))

    def plotTabBar_currentChanged(self, index):

        # clear the selection to prepare a new one
        self.ui.variableTreeWidget.clearSelection()
        for active_path_str in self.plot_items[index].canvas.active_paths.keys():
            path = active_path_str.split("/")
            item = self.ui.variableTreeWidget.topLevelItem(0)
            for subpath in path[1:]:
                for child_id in range(item.childCount()):
                    if item.child(child_id).text(0) == subpath:
                        item = item.child(child_id)
                        break
            item.setSelected(True)

    @pyqtSlot()
    def update_slider(self):
        if not self.slider_pressed:
            self.ui.timeSlider.setValue(self.signal_provider.index)
            self.ui.timeLabel.setText(f"{self.signal_provider.current_time:.2f}")
            self.text_logger.highlight_cell(
                self.find_text_log_index(self.get_text_log_item_path())
            )

    def closeEvent(self, event):
        # close the window
        self.pyconsole.close()
        for video_item in self.video_items:
            del video_item.media_player

        self.meshcat_provider.state = PeriodicThreadState.closed
        self.meshcat_provider.wait()

        self.signal_provider.state = PeriodicThreadState.closed
        self.signal_provider.wait()

        event.accept()

    def __populate_variable_tree_widget(self, obj, parent) -> QTreeWidgetItem:
        if not isinstance(obj, dict):
            return parent
        if "data" in obj.keys() and "timestamps" in obj.keys():
            temp_array = obj["data"]
            try:
                n_cols = temp_array.shape[1]
            except IndexError:
                # This happens in the case the variable is a scalar.
                n_cols = 1

            # In yarp telemetry v0.4.0 the elements_names was saved.
            if "elements_names" in obj.keys():
                for name in obj["elements_names"]:
                    item = QTreeWidgetItem([name])
                    parent.addChild(item)
            else:
                for i in range(n_cols):
                    item = QTreeWidgetItem(["Element " + str(i)])
                    parent.addChild(item)
            return parent
        for key, value in obj.items():
            item = QTreeWidgetItem([key])
            item = self.__populate_variable_tree_widget(value, item)
            item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
            parent.addChild(item)
        return parent

    def __populate_text_logging_tree_widget(self, obj, parent) -> QTreeWidgetItem:
        if not isinstance(obj, dict):
            return parent

        if "data" in obj.keys() and "timestamps" in obj.keys():
            return parent

        for key, value in obj.items():
            item = QTreeWidgetItem([key])
            item = self.__populate_text_logging_tree_widget(value, item)
            if "data" not in value.keys():
                item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
            parent.addChild(item)
        return parent

    def open_mat_file(self):
        file_name, _ = QFileDialog.getOpenFileName(
            self, "Open a mat file", ".", filter="*.mat"
        )
        if file_name:
            self.signal_provider.open_mat_file(file_name)
            self.signal_size = len(self.signal_provider)

            # populate tree
            root = list(self.signal_provider.data.keys())[0]
            root_item = QTreeWidgetItem([root])
            root_item.setFlags(root_item.flags() & ~Qt.ItemIsSelectable)
            items = self.__populate_variable_tree_widget(
                self.signal_provider.data[root], root_item
            )
            self.ui.variableTreeWidget.insertTopLevelItems(0, [items])

            # populate text logging tree
            root = list(self.signal_provider.text_logging_data.keys())[0]
            root_item = QTreeWidgetItem([root])
            root_item.setFlags(root_item.flags() & ~Qt.ItemIsSelectable)
            items = self.__populate_text_logging_tree_widget(
                self.signal_provider.text_logging_data[root], root_item
            )
            self.ui.yarpTextLogTreeWidget.insertTopLevelItems(0, [items])

            # spawn the console
            self.pyconsole.push_local_ns("data", self.signal_provider.data)

            self.ui.timeSlider.setMaximum(self.signal_size)
            self.ui.startButton.setEnabled(True)
            self.ui.timeSlider.setEnabled(True)

            # get all the video associated to the dataset

            filename_without_path = pathlib.Path(file_name).name
            (prefix, sep, suffix) = filename_without_path.rpartition(".")

            video_filenames = [
                str(pathlib.Path(file_name).parent.absolute() / pathlib.Path(f))
                for f in os.listdir(pathlib.Path(file_name).parent.absolute())
                if re.search(prefix + "_[a-zA-Z0-9_]*\.mp4$", f)
            ]

            # for every video we create a video item and we append it to the tab
            for video_filename in video_filenames:
                video_prefix, _, _ = pathlib.Path(video_filename).name.rpartition(".")
                video_label = str(video_prefix).replace(prefix + "_", "")
                self.video_items.append(VideoItem(video_filename=video_filename))
                self.ui.meshcatAndVideoTab.addTab(
                    self.video_items[-1], get_icon("videocam-outline.svg"), video_label
                )
                self.logger.write_to_log("Video '" + video_filename + "' opened.")
            # load the model
            self.meshcat_provider.load_model(
                self.signal_provider.joints_name, self.signal_provider.robot_name
            )

            self.meshcat_provider.state = PeriodicThreadState.running

            self.dataset_loaded = True

            # write something in the log
            self.logger.write_to_log("File '" + file_name + "' opened.")
            self.logger.write_to_log(
                "Robot name: '" + self.signal_provider.robot_name + "'."
            )

    def open_about(self):
        self.about.show()


class Logger:
    """
    Logger class shows events during the execution of the viewer.
    """

    def __init__(self, log_widget, scroll_area, add_time=True):
        # set log widget from main window
        self.log_widget = log_widget

        # set scroll area form main window
        self.scroll_area = scroll_area

        self.add_time = add_time

    def write_to_log(self, text, font_color=None, background_color=None):
        """
        Log the text "text" with a timestamp.
        """

        # extract current text from the log widget
        current_text = self.log_widget.text()

        if font_color is not None:
            text = '<font color="' + str(font_color) + '">' + text + "</font>"

        if background_color is not None:
            text = (
                '<span style="background-color:'
                + str(background_color)
                + '">'
                + text
                + "</span>"
            )

        # compose new text
        # convert local time to string
        if self.add_time:
            time_str = strftime(" [%H:%M:%S] ", localtime())
            #
            new_text = current_text + time_str + text + "<br>"
        else:
            new_text = current_text + text + "<br>"

        # log into the widget
        self.log_widget.setText(new_text)

        # scroll down text
        self.scroll_down()

    def clean(self):
        self.log_widget.clear()

    def scroll_down(self):
        """
        Scroll down the slider of the scroll area
        linked to this logger
        """
        # extract scroll bar from the scroll area
        scroll_bar = self.scroll_area.verticalScrollBar()

        # set maximum value of the slider, 1000 is enough
        scroll_bar.setMaximum(1000)
        scroll_bar.setValue(scroll_bar.maximum())


if __name__ == "__main__":
    # construct a QApplication
    app = QtWidgets.QApplication(sys.argv)

    # instantiate the main window and add the Matplotlib canvas
    gui = RobotViewerMainWindow()

    # show the main window
    gui.show()

    sys.exit(app.exec_())



# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

from setuptools import setup

if __name__ == "__main__":
    setup()



from PyQt5 import QtWidgets
from PyQt5.QtWidgets import (
    QTableWidgetItem,
)

from PyQt5.QtGui import QPainter, QColor, QFont, QBrush


class TextLoggingItem:
    def __init__(self, table_widget, background_color="#ffeba8"):
        self.table_widget = table_widget
        self.index_coloured_cell = None
        self.background_color = QBrush(QColor(background_color))
        self.white_background = QBrush(QColor("#ffffff"))
        self.table_widget.clear()

    def add_entry(self, text, timestamp, font_color=None):

        item = QTableWidgetItem(text)
        if font_color is not None:
            # text = '<font color="' + str(font_color) + '">' + text + "</font>"
            item.setForeground(QBrush(QColor(font_color)))

        item_timestamp = QTableWidgetItem(f"{timestamp:.2f}")
        if font_color is not None:
            # text = '<font color="' + str(font_color) + '">' + text + "</font>"
            item_timestamp.setForeground(QBrush(QColor(font_color)))

        self.table_widget.insertRow(self.table_widget.rowCount())
        self.table_widget.setItem(self.table_widget.rowCount() - 1, 0, item_timestamp)
        self.table_widget.setItem(self.table_widget.rowCount() - 1, 1, item)

    def clean(self):
        self.table_widget.setColumnCount(2)
        self.table_widget.setRowCount(0)
        self.table_widget.resizeColumnToContents(1)
        self.table_widget.clear()
        header = self.table_widget.horizontalHeader()
        header.setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.index_coloured_cell = None

    def highlight_cell(self, index):
        if index == self.index_coloured_cell:
            return

        if self.index_coloured_cell is not None:
            self.table_widget.item(self.index_coloured_cell, 0).setBackground(
                self.white_background
            )
            self.table_widget.item(self.index_coloured_cell, 1).setBackground(
                self.white_background
            )

        if index is not None:
            self.table_widget.item(index, 0).setBackground(self.background_color)
            self.table_widget.item(index, 1).setBackground(self.background_color)

        self.index_coloured_cell = index



#!/usr/bin/env python3

# Copyright (C) 2022 Istituto Italiano di Tecnologia (IIT). All rights reserved.
# This software may be modified and distributed under the terms of the
# Released under the terms of the BSD 3-Clause License

import sys
import os

# GUI
from robot_log_visualizer.ui.gui import RobotViewerMainWindow
from PyQt5.QtWidgets import QApplication

from robot_log_visualizer.file_reader.signal_provider import SignalProvider

# Meshcat
from robot_log_visualizer.robot_visualizer.meshcat_provider import MeshcatProvider


def main():
    thread_periods = {
        "meshcat_provider": 0.02,
        "signal_provider": 0.01,
        "plot_animation": 0.02,
    }

    # instantiate device_manager
    signal_provider = SignalProvider(period=thread_periods["signal_provider"])

    meshcat_provider = MeshcatProvider(
        period=thread_periods["meshcat_provider"], signal_provider=signal_provider
    )

    # instantiate a QApplication
    app = QApplication(sys.argv)

    # instantiate the main window
    gui = RobotViewerMainWindow(
        signal_provider=signal_provider,
        meshcat_provider=meshcat_provider,
        animation_period=thread_periods["plot_animation"],
    )

    # show the main window
    gui.show()

    signal_provider.start()
    meshcat_provider.start()

    return app.exec_()


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'robot_log_visualizer/ui/misc/plot_tab.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_PlotTab(object):
    def setupUi(self, PlotTab):
        PlotTab.setObjectName("PlotTab")
        PlotTab.resize(331, 149)
        self.horizontalLayout = QtWidgets.QHBoxLayout(PlotTab)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.plotLayout = QtWidgets.QVBoxLayout()
        self.plotLayout.setObjectName("plotLayout")
        self.horizontalLayout.addLayout(self.plotLayout)

        self.retranslateUi(PlotTab)
        QtCore.QMetaObject.connectSlotsByName(PlotTab)

    def retranslateUi(self, PlotTab):
        _translate = QtCore.QCoreApplication.translate
        PlotTab.setWindowTitle(_translate("PlotTab", "Form"))
